(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// server/pre-meteor.js                                                //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
// This file implements logic that we place in front of our main Meteor application,
// including routing of requests to proxies and handling of static web publishing.
                                                                       //
var Url = Npm.require("url");                                          // 20
var Fs = Npm.require("fs");                                            // 21
var Dns = Npm.require("dns");                                          // 22
var Promise = Npm.require("es6-promise").Promise;                      // 23
var Future = Npm.require("fibers/future");                             // 24
var Http = Npm.require("http");                                        // 25
                                                                       //
var HOSTNAME = Url.parse(process.env.ROOT_URL).hostname;               // 27
var DDP_HOSTNAME = process.env.DDP_DEFAULT_CONNECTION_URL && Url.parse(process.env.DDP_DEFAULT_CONNECTION_URL).hostname;
var CACHE_TTL_SECONDS = 30; // 30 seconds.  Cache-Control expects units of seconds, not millis.
var DNS_CACHE_TTL = CACHE_TTL_SECONDS * 1000; // DNS cache is in millis.
                                                                       //
var staticHandlers = {};                                               // 33
// Maps grain public IDs to Connect handlers.                          //
// TODO(perf): Garbage-collect this map?                               //
                                                                       //
var dnsCache = {};                                                     // 37
// Unfortunately, node's DNS library doesn't cache results, so we do our own caching.
// Unfortunately, node's DNS library also dosen't give us TTLs. So, we'll cache for
// DNS_CACHE_TTL (a relatively small value) and rely on the upstream DNS server to implement
// better caching.                                                     //
                                                                       //
function isSandstormShell(hostname) {                                  // 43
  // Is this hostname mapped to the Sandstorm shell?                   //
                                                                       //
  return hostname === HOSTNAME || DDP_HOSTNAME && hostname === DDP_HOSTNAME;
}                                                                      //
                                                                       //
var mime = Connect["static"].mime;                                     // 49
                                                                       //
function wwwHandlerForGrain(grainId) {                                 // 51
  return function (request, response, cb) {                            // 52
    var path = request.url;                                            // 53
                                                                       //
    // If a directory, open 'index.html'.                              //
    if (path.slice(-1) === "/") {                                      // 56
      path = path + "index.html";                                      // 57
    }                                                                  //
                                                                       //
    // Strip leading '/'.                                              //
    if (path[0] === "/") path = path.slice(1);                         // 61
                                                                       //
    // Strip query.                                                    //
    path = path.split("?")[0];                                         // 64
                                                                       //
    var type = mime.lookup(path);                                      // 66
    var charset = mime.charsets.lookup(type);                          // 67
    if (charset) {                                                     // 68
      type = type + "; charset=" + charset;                            // 69
    } else if (type === "application/json") {                          //
      // HACK: Apparently the MIME module does not assume UTF-8 for JSON. :(
      type = type + "; charset=utf-8";                                 // 72
    }                                                                  //
                                                                       //
    var started = false;                                               // 75
    var sawEnd = false;                                                // 76
                                                                       //
    // TODO(perf): Automatically gzip text content? Use Express's 'compress' middleware for this?
    //   Note that nginx will also auto-compress things...             //
                                                                       //
    var headers = {                                                    // 81
      "Content-Type": type,                                            // 82
      "Cache-Control": "public, max-age=" + CACHE_TTL_SECONDS          // 83
    };                                                                 //
                                                                       //
    if (path === "apps/index.json" || path.match(/apps\/[a-z0-9]{52}[.]json/) || path === "experimental/index.json" || path.match(/experimental\/[a-z0-9]{52}[.]json/)) {
      // TODO(cleanup): Extra special terrible hack: The app index needs to serve these JSON files
      //   cross-origin. We could almost just make all web sites allow cross-origin since generally
      //   web publishing is meant to publish public content. There is one case where this is
      //   problematic, though: sites behind a firewall. Those sites could potentially be read
      //   by outside sites if CORS is enabled on them. Some day we should make it so apps can
      //   explicitly opt-in to allowing cross-origin queries but that day is not today.
      headers["Access-Control-Allow-Origin"] = "*";                    // 96
    }                                                                  //
                                                                       //
    var stream = {                                                     // 99
      expectSize: function (size) {                                    // 100
        if (!started) {                                                // 101
          started = true;                                              // 102
          response.writeHead(200, _.extend(headers, { "Content-Length": size }));
        }                                                              //
      },                                                               //
                                                                       //
      write: function (data) {                                         // 107
        if (!started) {                                                // 108
          started = true;                                              // 109
          response.writeHead(200, headers);                            // 110
        }                                                              //
                                                                       //
        response.write(data);                                          // 113
      },                                                               //
                                                                       //
      done: function (data) {                                          // 116
        if (!started) {                                                // 117
          started = true;                                              // 118
          response.writeHead(200, _.extend(headers, { "Content-Length": 0 }));
        }                                                              //
                                                                       //
        sawEnd = true;                                                 // 122
        response.end();                                                // 123
      }                                                                //
    };                                                                 //
                                                                       //
    globalBackend.useGrain(grainId, function (supervisor) {            // 127
      return supervisor.getWwwFileHack(path, stream).then(function (result) {
        // jscs:disable disallowQuotedKeysInObjects                    //
        var status = result.status;                                    // 130
        if (status === "file") {                                       // 131
          if (!sawEnd) {                                               // 132
            console.error("getWwwFileHack didn't write file to stream");
            if (!started) {                                            // 134
              response.writeHead(500, {                                // 135
                "Content-Type": "text/plain"                           // 136
              });                                                      //
              response.end("Internal server error");                   // 138
            }                                                          //
                                                                       //
            response.end();                                            // 141
          }                                                            //
        } else if (status === "directory") {                           //
          if (started) {                                               // 144
            console.error("getWwwFileHack wrote to stream for directory");
            if (!sawEnd) {                                             // 146
              response.end();                                          // 147
            }                                                          //
          } else {                                                     //
            response.writeHead(303, {                                  // 150
              "Content-Type": "text/plain",                            // 151
              "Location": "/" + path + "/",                            // 152
              "Cache-Control": "public, max-age=" + CACHE_TTL_SECONDS  // 153
            });                                                        //
            response.end("redirect: /" + path + "/");                  // 155
          }                                                            //
        } else if (status === "notFound") {                            //
          if (started) {                                               // 158
            console.error("getWwwFileHack wrote to stream for notFound");
            if (!sawEnd) {                                             // 160
              response.end();                                          // 161
            }                                                          //
          } else {                                                     //
            response.writeHead(404, {                                  // 164
              "Content-Type": "text/plain"                             // 165
            });                                                        //
            response.end("404 not found: /" + path);                   // 167
          }                                                            //
        } else {                                                       //
          console.error("didn't understand result of getWwwFileHack:", status);
          if (!started) {                                              // 171
            response.writeHead(500, {                                  // 172
              "Content-Type": "text/plain"                             // 173
            });                                                        //
            response.end("Internal server error");                     // 175
          }                                                            //
        }                                                              //
      });                                                              //
    })["catch"](function (err) {                                       //
      console.error(err.stack);                                        // 180
    });                                                                //
  };                                                                   //
}                                                                      //
                                                                       //
function writeErrorResponse(res, err) {                                // 185
  var status = 500;                                                    // 186
  if (err instanceof Meteor.Error && typeof err.error === "number" && err.error >= 400 && err.error < 600) {
    status = err.error;                                                // 189
  } else if (err.httpErrorCode) {                                      //
    status = err.httpErrorCode;                                        // 191
  }                                                                    //
                                                                       //
  // Log errors that are our fault, but not errors that are the client's fault.
  if (status >= 500) console.error(err.stack);                         // 195
                                                                       //
  res.writeHead(status, { "Content-Type": err.htmlMessage ? "text/html" : "text/plain" });
  res.end(err.htmlMessage || err.message);                             // 198
}                                                                      //
                                                                       //
var PNG_MAGIC = new Buffer([0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A]);
var JPEG_MAGIC = new Buffer([0xFF, 0xD8, 0xFF]);                       // 202
                                                                       //
function checkMagic(buf, magic) {                                      // 204
  if (buf.length < magic.length) return false;                         // 205
                                                                       //
  for (var i = 0; i < magic.length; i++) {                             // 207
    if (buf[i] != magic[i]) return false;                              // 208
  }                                                                    //
                                                                       //
  return true;                                                         // 211
}                                                                      //
                                                                       //
function serveSelfTest(req, res) {                                     // 214
  try {                                                                // 215
    if (req.method === "GET" && req.url === "/") {                     // 216
      var content = new Buffer("Self-test OK.");                       // 218
                                                                       //
      // Convert the ROOT_URL to something that is a valid origin.     //
      var rootUrlAsOrigin = process.env.ROOT_URL;                      // 221
      if (rootUrlAsOrigin.slice(-1) === "/") {                         // 222
        rootUrlAsOrigin = rootUrlAsOrigin.slice(0, -1);                // 223
      }                                                                //
                                                                       //
      res.writeHead(200, {                                             // 226
        "Content-Type": "text/plain",                                  // 227
        "Content-Length": content.length,                              // 228
        "Access-Control-Allow-Origin": rootUrlAsOrigin                 // 229
      });                                                              //
      res.end(content);                                                // 231
    } else {                                                           //
      res.writeHead(400, { "Content-Type": "text/plain" });            // 233
      res.end("Bad request to self-test subdomain.");                  // 234
      return;                                                          // 235
    }                                                                  //
  } catch (err) {                                                      //
    console.log("WARNING: An error occurred while serving self-test; proceeding anyway:", err);
  }                                                                    //
}                                                                      //
                                                                       //
function serveStaticAsset(req, res) {                                  // 242
  inMeteor(function () {                                               // 243
    if (req.method === "GET") {                                        // 244
      // jscs:disable validateQuoteMarks                               //
      var assetCspHeader = "default-src 'none'; style-src 'unsafe-inline'; sandbox";
      if (req.headers['if-none-match'] === 'permanent') {              // 247
        // Cache never invalidates since we use a new URL for every resource.
        res.writeHead(304, {                                           // 249
          'Cache-Control': 'public, max-age=31536000',                 // 250
          'ETag': 'permanent',                                         // 251
                                                                       //
          // To be safe, send these again, although it shouldn't be necessary.
          'Content-Security-Policy': assetCspHeader,                   // 254
          'Access-Control-Allow-Origin': '*',                          // 255
          'X-Content-Type-Options': 'nosniff'                          // 256
        });                                                            //
        res.end();                                                     // 258
        return;                                                        // 259
      }                                                                //
                                                                       //
      var url = Url.parse(req.url);                                    // 262
      var asset = globalDb.getStaticAsset(url.pathname.slice(1));      // 263
                                                                       //
      if (asset) {                                                     // 265
        var headers = {                                                // 266
          'Content-Type': asset.mimeType,                              // 267
          'Content-Length': asset.content.length,                      // 268
                                                                       //
          // Assets can be cached forever because each one has a unique ID.
          'Cache-Control': 'public, max-age=31536000',                 // 271
                                                                       //
          // Since different resources get different URLs, we can use a static etag.
          'ETag': 'permanent',                                         // 274
                                                                       //
          // Set strict Content-Security-Policy to prevent static assets from executing any script
          // or doing basically anything when browsed to directly. The static assets host is not
          // intended to serve HTML. Mostly, it serves images and javascript -- note that setting
          // the CSP header on Javascript files does not prevent other hosts from voluntarily
          // specifying these scripts in <script> tags.                //
          'Content-Security-Policy': assetCspHeader,                   // 281
                                                                       //
          // Allow any host to fetch these assets. This is safe since requests to this host are
          // totally side-effect-free and the asset ID acts as a capability to prevent loading
          // assets you're not supposed to know about.                 //
          'Access-Control-Allow-Origin': '*',                          // 286
                                                                       //
          // Extra protection against content type trickery.           //
          'X-Content-Type-Options': 'nosniff'                          // 289
        };                                                             //
                                                                       //
        if (asset.encoding) {                                          // 292
          headers['Content-Encoding'] = asset.encoding;                // 293
        }                                                              //
                                                                       //
        res.writeHead(200, headers);                                   // 296
        res.end(asset.content);                                        // 297
      } else {                                                         //
        res.writeHead(404, { 'Content-Type': 'text/plain' });          // 299
        res.end('not found');                                          // 300
      }                                                                //
    } else if (req.method === 'POST') {                                //
      var _ret = (function () {                                        //
        res.setHeader('Access-Control-Allow-Origin', '*');             // 303
                                                                       //
        var url = Url.parse(req.url);                                  // 305
        var purpose = globalDb.fulfillAssetUpload(url.pathname.slice(1));
        if (!purpose) {                                                // 307
          res.writeHead(404, { 'Content-Type': 'text/plain' });        // 308
          res.end('Upload token not found or expired.');               // 309
          return {                                                     // 310
            v: undefined                                               //
          };                                                           //
        }                                                              //
                                                                       //
        var userId = purpose.profilePicture.userId;                    // 313
        var identityId = purpose.profilePicture.identityId;            // 314
        check(userId, String);                                         // 315
        check(identityId, String);                                     // 316
                                                                       //
        var buffers = [];                                              // 318
        var totalSize = 0;                                             // 319
        var done = new Future();                                       // 320
        req.on('data', function (buf) {                                // 321
          totalSize += buf.length;                                     // 322
          if (totalSize <= 64 * 1024) {                                // 323
            buffers.push(buf);                                         // 324
          }                                                            //
        });                                                            //
        req.on('end', done["return"].bind(done));                      // 327
        req.on('error', done["throw"].bind(done));                     // 328
        done.wait();                                                   // 329
                                                                       //
        if (totalSize > 64 * 1024) {                                   // 331
          // TODO(soon): Resize the image ourselves.                   //
          res.writeHead(400, { 'Content-Type': 'text/plain' });        // 333
          res.end('Picture too large; please use an image under 64 KiB.');
          return {                                                     // 335
            v: undefined                                               //
          };                                                           //
        }                                                              //
                                                                       //
        var content = Buffer.concat(buffers);                          // 338
        var type = undefined;                                          // 339
        if (checkMagic(content, PNG_MAGIC)) {                          // 340
          type = 'image/png';                                          // 341
        } else if (checkMagic(content, JPEG_MAGIC)) {                  //
          type = 'image/jpeg';                                         // 343
        } else {                                                       //
          res.writeHead(400, { 'Content-Type': 'text/plain' });        // 345
          res.end('Image must be PNG or JPEG.');                       // 346
          return {                                                     // 347
            v: undefined                                               //
          };                                                           //
        }                                                              //
                                                                       //
        var assetId = globalDb.addStaticAsset({ mimeType: type }, content);
        var old = Meteor.users.findAndModify({                         // 351
          query: { '_id': identityId },                                // 352
          update: { $set: { 'profile.picture': assetId } },            // 353
          fields: { 'profile.picture': 1 }                             // 354
        });                                                            //
                                                                       //
        if (old && old.profile && old.profile.picture) {               // 357
          globalDb.unrefStaticAsset(old.profile.picture);              // 358
        }                                                              //
                                                                       //
        res.writeHead(204, {});                                        // 361
        res.end();                                                     // 362
      })();                                                            //
                                                                       //
      if (typeof _ret === "object") return _ret.v;                     //
    } else if (req.method === 'OPTIONS') {                             //
      var requestedHeaders = req.headers['access-control-request-headers'];
      if (requestedHeaders) {                                          // 365
        res.setHeader('Access-Control-Allow-Headers', requestedHeaders);
      }                                                                //
                                                                       //
      res.writeHead(204, {                                             // 369
        'Access-Control-Allow-Origin': '*',                            // 370
        'Access-Control-Allow-Methods': 'GET, HEAD, POST, OPTIONS',    // 371
        'Access-Control-Max-Age': '3600'                               // 372
      });                                                              //
      res.end();                                                       // 374
    } else {                                                           //
      res.writeHead(405, 'Method Not Allowed', {                       // 376
        'Allow': 'GET, POST, OPTIONS',                                 // 377
        'Content-Type': 'text/plain'                                   // 378
      });                                                              //
      res.end('405 Method Not Allowed: ' + req.method);                // 380
    }                                                                  //
  })["catch"](function (err) {                                         //
    writeErrorResponse(res, err);                                      // 383
  });                                                                  //
}                                                                      //
                                                                       //
Meteor.startup(function () {                                           // 387
                                                                       //
  var meteorUpgradeListeners = WebApp.httpServer.listeners('upgrade');
  WebApp.httpServer.removeAllListeners('upgrade');                     // 390
                                                                       //
  WebApp.httpServer.on('upgrade', function (req, socket, head) {       // 392
    Promise.resolve(undefined).then(function () {                      // 393
      if (!req.headers.host) {                                         // 394
        throw new Meteor.Error(400, 'Missing Host header');            // 395
      } else if (isSandstormShell(req.headers.host.split(':')[0])) {   //
        // Go on to Meteor.                                            //
        for (var ii = 0; ii < meteorUpgradeListeners.length; ++ii) {   // 398
          meteorUpgradeListeners[ii](req, socket, head);               // 399
        }                                                              //
                                                                       //
        return true;                                                   // 402
      } else {                                                         //
        var id = matchWildcardHost(req.headers.host);                  // 404
        if (id) {                                                      // 405
          return tryProxyUpgrade(id, req, socket, head);               // 406
        } else {                                                       //
          return false;                                                // 408
        }                                                              //
      }                                                                //
    }).then(function (handled) {                                       //
      if (!handled) socket.destroy();                                  // 412
    })["catch"](function (err) {                                       //
      console.error('WebSocket event handler failed:', err.stack);     // 414
      socket.destroy();                                                // 415
    });                                                                //
  });                                                                  //
                                                                       //
  // BlackrockPayments is only defined in the Blackrock build of Sandstorm.
  if (global.BlackrockPayments) {                                      // 420
    // Have to check with global, because it could be undefined.       //
    WebApp.rawConnectHandlers.use(BlackrockPayments.makeConnectHandler(globalDb));
  }                                                                    //
                                                                       //
  // This function serves responses on Sandstorm's main HTTP/HTTPS     //
  // port.                                                             //
  var serveMeteorOrStaticPublishing = function (req, res, next) {      // 426
    return dispatchToMeteorOrStaticPublishing(req, res, next, false);  // 427
  };                                                                   //
                                                                       //
  // "Alternate ports" are ports other than the main HTTP or HTTPS     //
  // port. For requests to the shell & grains, we redirect to the main
  // port. For static publishing, we serve it.                         //
  //                                                                   //
  // They are bound to FD #4 and higher.                               //
                                                                       //
  function getNumberOfAlternatePorts() {                               // 436
    var numPorts = process.env.PORT.split(',').length;                 // 437
    var numAlternatePorts = numPorts - 1;                              // 438
    return numAlternatePorts;                                          // 439
  };                                                                   //
                                                                       //
  var canonicalizeShellOrWildcardUrl = function (hostname, url) {      // 442
    // Start with ROOT_URL, apply host & path from inbound URL, then   //
    // redirect.                                                       //
    var targetUrl = Url.parse(process.env.ROOT_URL);                   // 445
                                                                       //
    // Retain the protocol & port from ROOT_URL but use the inbound    //
    // hostname.                                                       //
    targetUrl.host = hostname + ':' + targetUrl.port;                  // 449
                                                                       //
    // The following allows to avoid decoding + re-encoding query      //
    // string parameters, if provided.                                 //
    targetUrl = Url.resolve(Url.format(targetUrl), url);               // 453
    return targetUrl;                                                  // 454
  };                                                                   //
                                                                       //
  var redirectToMeteorOrServeStaticPublishing = function (req, res, next) {
    return dispatchToMeteorOrStaticPublishing(req, res, next, true);   // 458
  };                                                                   //
                                                                       //
  for (var i = 0; i < getNumberOfAlternatePorts(); i++) {              // 461
    // Call createServerForSandstorm() to skip our monkey patching.    //
    var alternatePortServer = Http.createServerForSandstorm(redirectToMeteorOrServeStaticPublishing);
    alternatePortServer.listen({ fd: i + 4 });                         // 464
  }                                                                    //
                                                                       //
  var dispatchToMeteorOrStaticPublishing = function (req, res, next, redirectRatherThanServeShell) {
    if (!req.headers.host) {                                           // 468
      res.writeHead(400, { 'Content-Type': 'text/plain' });            // 469
      res.end('Missing Host header');                                  // 470
      return;                                                          // 471
    }                                                                  //
                                                                       //
    var hostname = req.headers.host.split(':')[0];                     // 474
    if (isSandstormShell(hostname)) {                                  // 475
      // Go on to Meteor, or serve a redirect.                         //
      if (redirectRatherThanServeShell) {                              // 477
        res.writeHead(302, { 'Location': canonicalizeShellOrWildcardUrl(hostname, req.url) });
        res.end();                                                     // 479
        return;                                                        // 480
      } else {                                                         //
        return next();                                                 // 482
      }                                                                //
    }                                                                  //
                                                                       //
    // This is not our main host. See if it's a member of the wildcard.
    var publicIdPromise = undefined;                                   // 487
                                                                       //
    var id = matchWildcardHost(req.headers.host);                      // 489
    if (id) {                                                          // 490
      // Match!                                                        //
      if (redirectRatherThanServeShell) {                              // 492
        res.writeHead(302, { 'Location': canonicalizeShellOrWildcardUrl(hostname, req.url) });
        res.end();                                                     // 494
        return;                                                        // 495
      }                                                                //
                                                                       //
      if (id === 'static') {                                           // 498
        // Static assets domain.                                       //
        serveStaticAsset(req, res);                                    // 500
        return;                                                        // 501
      }                                                                //
                                                                       //
      if (id.match(/^selftest-/)) {                                    // 504
        // Self test domain pattern. Starts w/ hyphen to avoid ambiguity with grain session/static
        // publishing wildcard hosts.                                  //
        serveSelfTest(req, res);                                       // 507
        return;                                                        // 508
      }                                                                //
                                                                       //
      // Try to route the request to a session.                        //
      publicIdPromise = tryProxyRequest(id, req, res).then(function (handled) {
        if (handled) {                                                 // 513
          return null;                                                 // 514
        } else {                                                       //
          return id;                                                   // 516
        }                                                              //
      });                                                              //
    } else {                                                           //
      // Not a wildcard host. Perhaps it is a custom host.             //
      publicIdPromise = lookupPublicIdFromDns(hostname);               // 521
    }                                                                  //
                                                                       //
    publicIdPromise.then(function (publicId) {                         // 524
      if (publicId) {                                                  // 525
        return Promise.resolve(undefined).then(function () {           // 526
          var handler = staticHandlers[publicId];                      // 527
          if (handler) {                                               // 528
            return handler;                                            // 529
          } else {                                                     //
            // We don't have a handler for this publicId, so look it up in the grain DB.
            return inMeteor(function () {                              // 532
              var grain = Grains.findOne({ publicId: publicId }, { fields: { _id: 1 } });
              if (!grain) {                                            // 534
                throw new Meteor.Error(404, 'No such grain for public ID: ' + publicId);
              }                                                        //
                                                                       //
              var grainId = grain._id;                                 // 538
              return staticHandlers[publicId] = wwwHandlerForGrain(grainId);
            });                                                        //
          }                                                            //
        }).then(function (handler) {                                   //
          handler(req, res, function (err) {                           // 543
            if (err) {                                                 // 544
              next(err);                                               // 545
            } else {                                                   //
              res.writeHead(404, { 'Content-Type': 'text/plain' });    // 547
              res.end('404 not found: ' + req.url);                    // 548
            }                                                          //
          });                                                          //
        });                                                            //
      } else {                                                         //
        return Promise.resolve(undefined);                             // 553
      }                                                                //
    })["catch"](function (err) {                                       //
      writeErrorResponse(res, err);                                    // 556
    });                                                                //
  };                                                                   //
                                                                       //
  WebApp.rawConnectHandlers.use(serveMeteorOrStaticPublishing);        // 560
});                                                                    //
                                                                       //
var errorTxtMapping = {};                                              // 563
errorTxtMapping[Dns.NOTFOUND] = '<p>' + 'If you were trying to configure static publishing for a blog or website, powered ' + 'by a Sandstorm app hosted at this server, you either have not added DNS TXT records ' + 'correctly, or the DNS cache has not updated yet (may take a while, like 5 minutes to one ' + 'hour).</p>';
errorTxtMapping[Dns.NODATA] = errorTxtMapping[Dns.NOTFOUND];           // 569
errorTxtMapping[Dns.TIMEOUT] = '<p>' + 'The DNS query has timed out, which may be a sign of poorly configured DNS on the server.</p>';
errorTxtMapping[Dns.CONNREFUSED] = '<p>' + 'The DNS server refused the connection, which means either your DNS server is down/unreachable, ' + 'or the server has misconfigured their DNS.</p>';
                                                                       //
function lookupPublicIdFromDns(hostname) {                             // 576
  // Given a hostname, determine its public ID.                        //
  // We look for a TXT record indicating the public ID. Unfortunately, according to spec, a single
  // hostname cannot have both a CNAME and a TXT record, because a TXT lookup on a CNAME'd host
  // should actually be redirected to the CNAME, just like an A lookup would be. In practice lots
  // of DNS software actually allows TXT records on CNAMEs, and it seems to work, but some software
  // does not allow it and it's explicitly disallowed by the spec. Therefore, we instead look for
  // the TXT record on a subdomain.                                    //
  //                                                                   //
  // I also considered having the CNAME itself point to <publicId>.<hostname>, where
  // *.<hostname> is in turn a CNAME for the Sandstorm server. This approach seemed elegant at
  // first, but has a number of problems, the biggest being that it breaks the ability to place a
  // CDN like CloudFlare in front of the site.                         //
                                                                       //
  var cache = dnsCache[hostname];                                      // 590
  if (cache && Date.now() < cache.expiration) {                        // 591
    return Promise.resolve(cache.value);                               // 592
  }                                                                    //
                                                                       //
  return new Promise(function (resolve, reject) {                      // 595
    Dns.resolveTxt('sandstorm-www.' + hostname, function (err, records) {
      if (err) {                                                       // 597
        var errorMsg = errorTxtMapping[err.code] || '';                // 598
        var error = new Error('Error looking up DNS TXT records for host "' + hostname + '": ' + err.message);
        error.htmlMessage = '<style type="text/css">h2, h3, p { max-width: 600px; }</style>' + '<h2>Sandstorm static publishing needs further configuration (or wrong URL)</h2>' + errorMsg + '<p>To visit this Sandstorm server\'s main interface, go to: <a href=\'' + process.env.ROOT_URL + '\'>' + process.env.ROOT_URL + '</a></p>' + '<h3>DNS details</h3>' + '<p>Error looking up DNS TXT records for host "' + hostname + '": ' + err.message + '</p>' + '<p>If you have the <tt>dig</tt> tool, you can run this command to learn more:</p>' + '<p><tt>dig TXT sandstorm-www.' + hostname + '</tt></p>' + '<h3>Changing the server URL, or troubleshooting OAuth login</h3>' + '<p>If you are the server admin and want to use this address as the main interface, ' + 'edit /opt/sandstorm/sandstorm.conf, modify the BASE_URL setting, and restart ' + 'Sandstorm.</p>' + '<p>If you got here after trying to log in via OAuth (e.g. through GitHub or Google), ' + 'the problem is probably that the OAuth callback URL was set wrong. You need to ' + 'update it through the respective login provider\'s management console. The ' + 'easiest way to do that is to run <tt>sudo sandstorm admin-token</tt>, then ' + 'reconfigure the OAuth provider.</p>';
        error.httpErrorCode = _.contains(['ENOTFOUND', 'ENODATA'], err.code) ? 404 : 500;
        reject(error);                                                 // 621
      } else if (records.length !== 1) {                               //
        reject(new Error('Host "sandstorm-www.' + hostname + '" must have exactly one TXT record.'));
      } else {                                                         //
        var result = records[0];                                       // 625
        dnsCache[hostname] = { value: result, expiration: Date.now() + DNS_CACHE_TTL };
        resolve(result);                                               // 627
      }                                                                //
    });                                                                //
  });                                                                  //
}                                                                      //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=pre-meteor.js.map
