(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// server/stats-server.js                                              //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var DAY_MS = 24 * 60 * 60 * 1000;                                      // 17
                                                                       //
if (Mongo.Collection.prototype.aggregate) {                            // 19
  throw new Error("Looks like Meteor wrapped the Collection.aggregate() call. Make sure it " + "works then delete our own wrapper.");
}                                                                      //
                                                                       //
Mongo.Collection.prototype.aggregate = function () {                   // 24
  // Meteor doesn't wrapp Mongo's aggregate() method.                  //
  var raw = this.rawCollection();                                      // 26
  return Meteor.wrapAsync(raw.aggregate, raw).apply(raw, arguments);   // 27
};                                                                     //
                                                                       //
computeStats = function (since) {                                      // 30
  // We'll need this for a variety of queries.                         //
  var timeConstraint = { $gt: since };                                 // 32
                                                                       //
  // This calculates the number of user accounts that have been used   //
  // during the requested time period.                                 //
  var currentlyActiveUsersCount = Meteor.users.find({ expires: { $exists: false }, loginIdentities: { $exists: true },
    lastActive: timeConstraint }).count();                             // 38
                                                                       //
  // This calculates the number of grains that have been used during   //
  // the requested time period.                                        //
  var activeGrainsCount = Grains.find({ lastUsed: timeConstraint }).count();
                                                                       //
  // If Meteor.settings.allowDemoAccounts is true, DeleteStats         //
  // contains records of type `user` and `appDemoUser`, indicating     //
  // the number of those types of accounts that were created and       //
  // then auto-expired through the demo mode's auto-account-expiry.    //
  var deletedDemoUsersCount = DeleteStats.find({ type: "demoUser", lastActive: timeConstraint }).count();
  var deletedAppDemoUsersCount = DeleteStats.find({ type: "appDemoUser", lastActive: timeConstraint }).count();
                                                                       //
  // Similarly, if the demo is enabled, we auto-delete grains; we store that
  // fact in DeleteStats with type: "grain".                           //
  var deletedGrainsCount = DeleteStats.find({ type: "grain", lastActive: timeConstraint }).count();
                                                                       //
  var apps = Grains.aggregate([{ $match: { lastUsed: timeConstraint } }, { $group: {
      _id: "$appId",                                                   // 61
      grains: { $sum: 1 },                                             // 62
      userIds: { $addToSet: "$userId" }                                // 63
    }                                                                  //
  }, { $project: {                                                     //
      grains: 1,                                                       // 67
      owners: { $size: "$userIds" }                                    // 68
    }                                                                  //
  }]);                                                                 //
  apps = _.indexBy(apps, "_id");                                       // 72
                                                                       //
  for (var appId in babelHelpers.sanitizeForInObject(apps)) {          // 74
    // We need to count ApiTokens, which don't have appId denormalized into them. We therefore
    // have to fetch a list of grainIds first.                         //
    // TODO(perf): If stats are getting slow, denormalize appId into ApiTokens. Note that it is
    //   already denormalized for the specific case of apps that don't have icons, but the data
    //   for that use case is NOT safe to use here because it's intended that we might allow an
    //   app to mimic another app's icon by spoofing that app ID. In other words, the existing
    //   denormalization of appId should be considered "app ID for identicon purposes only". We'll
    //   need to add a new denormalization for stats purposes -- and make sure that it is not
    //   revealed to the user.                                         //
    var app = apps[appId];                                             // 84
    delete app._id;                                                    // 85
    var grains = Grains.find({                                         // 86
      lastUsed: timeConstraint,                                        // 87
      appId: appId                                                     // 88
    }, {                                                               //
      fields: { _id: 1 }                                               // 90
    }).fetch();                                                        //
    var grainIds = _.pluck(grains, "_id");                             // 92
                                                                       //
    var counts = ApiTokens.aggregate([{ $match: {                      // 94
        "owner.user.lastUsed": timeConstraint,                         // 96
        grainId: { $in: grainIds }                                     // 97
      }                                                                //
    }, { $group: { _id: "$owner.user.identityId" } }, { $group: {      //
        _id: "count",                                                  // 102
        count: { $sum: 1 }                                             // 103
      }                                                                //
    }]);                                                               //
                                                                       //
    if (counts.length > 0) {                                           // 108
      if (counts.length !== 1) {                                       // 109
        console.error("error: sharedUsers aggregation returned multiple rows");
      }                                                                //
                                                                       //
      app.sharedUsers = counts[0].count;                               // 113
    }                                                                  //
  }                                                                    //
                                                                       //
  // Count per-app appdemo users and deleted grains.                   //
  DeleteStats.aggregate([{ $match: {                                   // 118
      lastActive: timeConstraint,                                      // 120
      appId: { $exists: true }                                         // 121
    }                                                                  //
  }, { $group: {                                                       //
      _id: {                                                           // 125
        appId: "$appId",                                               // 126
        type: "$type"                                                  // 127
      },                                                               //
      count: { $sum: 1 }                                               // 129
    }                                                                  //
  }]).forEach(function (deletion) {                                    //
    var app = apps[deletion._id.appId];                                // 133
    if (!app) {                                                        // 134
      app = apps[deletion.appId] = {};                                 // 135
    }                                                                  //
                                                                       //
    if (deletion._id.type === "appDemoUser") {                         // 138
      app.appDemoUsers = deletion.count;                               // 139
    } else if (deletion._id.type === "grain") {                        //
      app.deleted = deletion.count;                                    // 141
    } else if (deletion._id.type === "demoGrain") {                    //
      app.demoed = deletion.count;                                     // 143
    }                                                                  //
  });                                                                  //
                                                                       //
  return {                                                             // 147
    activeUsers: currentlyActiveUsersCount,                            // 148
    demoUsers: deletedDemoUsersCount,                                  // 149
    appDemoUsers: deletedAppDemoUsersCount,                            // 150
    activeGrains: activeGrainsCount + deletedGrainsCount,              // 151
    apps: apps                                                         // 152
  };                                                                   //
};                                                                     //
                                                                       //
function recordStats() {                                               // 156
  var now = new Date();                                                // 157
                                                                       //
  var planStats = _.countBy(Meteor.users.find({ expires: { $exists: false }, "payments.id": { $exists: true } }, { fields: { plan: 1 } }).fetch(), "plan");
                                                                       //
  var record = {                                                       // 165
    timestamp: now,                                                    // 166
    daily: computeStats(new Date(now.getTime() - DAY_MS)),             // 167
    weekly: computeStats(new Date(now.getTime() - 7 * DAY_MS)),        // 168
    monthly: computeStats(new Date(now.getTime() - 30 * DAY_MS)),      // 169
    forever: computeStats(new Date(0)),                                // 170
    plans: planStats                                                   // 171
  };                                                                   //
  record.computeTime = Date.now() - now;                               // 173
                                                                       //
  ActivityStats.insert(record);                                        // 175
  var age = ActivityStats.find().count();                              // 176
  if (age > 3) {                                                       // 177
    var reportSetting = Settings.findOne({ _id: "reportStats" });      // 178
    if (!reportSetting) {                                              // 179
      // Setting not set yet, send out notifications and set it to false
      globalDb.sendAdminNotification("You can help Sandstorm by sending us some anonymous " + "usage stats. Click here for more info.", "/admin/stats");
      Settings.insert({ _id: "reportStats", value: "unset" });         // 183
    } else if (reportSetting.value === true) {                         //
      // The stats page which the user agreed we can send actually displays the whole history
      // of the server, but we're only sending stats from the last day. Let's also throw in the
      // length of said history. This is still strictly less information than what the user said
      // we're allowed to send.                                        //
      record.serverAge = age;                                          // 189
                                                                       //
      HTTP.post("https://alpha-api.sandstorm.io/data", {               // 191
        data: record,                                                  // 192
        headers: {                                                     // 193
          Authorization: "Bearer aT-mGyNwsgwZBbZvd5FWr0Ma79O9IehI4NiEO94y_oR",
          "Content-Type": "application/json"                           // 195
        }                                                              //
      });                                                              //
    }                                                                  //
  }                                                                    //
}                                                                      //
                                                                       //
if (!Meteor.settings.replicaNumber) {                                  // 202
  // Wait until 10:00 UTC (2:00 PST / 5:00 EST), then start recording stats every 24 hours.
  // (Only on the first replica to avoid conflicts.)                   //
  Meteor.setTimeout(function () {                                      // 205
    Meteor.setInterval(function () {                                   // 206
      recordStats();                                                   // 207
    }, DAY_MS);                                                        //
                                                                       //
    recordStats();                                                     // 210
  }, DAY_MS - (Date.now() - 10 * 60 * 60 * 1000) % DAY_MS);            //
                                                                       //
  Meteor.startup(function () {                                         // 213
    if (StatsTokens.find().count() === 0) {                            // 214
      StatsTokens.remove({});                                          // 215
      StatsTokens.insert({ _id: Random.id(22) });                      // 216
    }                                                                  //
  });                                                                  //
}                                                                      //
                                                                       //
Meteor.methods({                                                       // 221
  regenerateStatsToken: function () {                                  // 222
    if (!isAdmin()) {                                                  // 223
      throw new Meteor.Error(403, "Unauthorized", "User must be admin");
    }                                                                  //
                                                                       //
    StatsTokens.remove({});                                            // 227
    var token = StatsTokens.insert({ _id: Random.id(22) });            // 228
    return token._id;                                                  // 229
  }                                                                    //
});                                                                    //
                                                                       //
Router.map(function () {                                               // 233
  this.route("fetchStats", {                                           // 234
    where: "server",                                                   // 235
    path: "/fetchStats/:tokenId",                                      // 236
    action: function () {                                              // 237
      var token = StatsTokens.findOne({ _id: this.params.tokenId });   // 238
                                                                       //
      if (!token) {                                                    // 240
        this.response.writeHead(404, {                                 // 241
          "Content-Type": "text/plain"                                 // 242
        });                                                            //
        this.response.write("Token not found");                        // 244
        return this.response.end();                                    // 245
      }                                                                //
                                                                       //
      try {                                                            // 248
        var stats = ActivityStats.find().fetch();                      // 249
        var statsString = JSON.stringify(stats);                       // 250
                                                                       //
        this.response.writeHead(200, {                                 // 252
          "Content-Type": "application/json"                           // 253
        });                                                            //
        this.response.write(statsString);                              // 255
      } catch (error) {                                                //
        console.error(error.stack);                                    // 257
        this.response.writeHead(500, {                                 // 258
          "Content-Type": "text/plain"                                 // 259
        });                                                            //
        this.response.write(error.stack);                              // 261
      }                                                                //
                                                                       //
      return this.response.end();                                      // 264
    }                                                                  //
  });                                                                  //
});                                                                    //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=stats-server.js.map
