(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var Log = Package.logging.Log;
var _ = Package.underscore._;
var DDP = Package['ddp-client'].DDP;
var DDPServer = Package['ddp-server'].DDPServer;
var EJSON = Package.ejson.EJSON;
var Follower = Package['follower-livedata'].Follower;

/* Package-scope variables */
var AppConfig;

(function(){

///////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                           //
// packages/application-configuration/packages/application-configuration.js                  //
//                                                                                           //
///////////////////////////////////////////////////////////////////////////////////////////////
                                                                                             //
(function () {                                                                               // 1
                                                                                             // 2
////////////////////////////////////////////////////////////////////////////////////////     // 3
//                                                                                    //     // 4
// packages/application-configuration/config.js                                       //     // 5
//                                                                                    //     // 6
////////////////////////////////////////////////////////////////////////////////////////     // 7
                                                                                      //     // 8
var Future = Npm.require("fibers/future");                                            // 1   // 9
                                                                                      // 2   // 10
AppConfig = {};                                                                       // 3   // 11
                                                                                      // 4   // 12
                                                                                      // 5   // 13
AppConfig.findGalaxy = _.once(function () {                                           // 6   // 14
  if (!('GALAXY' in process.env || 'ULTRAWORLD_DDP_ENDPOINT' in process.env)) {       // 7   // 15
    return null;                                                                      // 8   // 16
  }                                                                                   // 9   // 17
  return Follower.connect(process.env.ULTRAWORLD_DDP_ENDPOINT || process.env.GALAXY); // 10  // 18
});                                                                                   // 11  // 19
                                                                                      // 12  // 20
var ultra = AppConfig.findGalaxy();                                                   // 13  // 21
                                                                                      // 14  // 22
var subFuture = new Future();                                                         // 15  // 23
var subFutureJobs = new Future();                                                     // 16  // 24
if (ultra) {                                                                          // 17  // 25
  ultra.subscribe("oneApp", process.env.GALAXY_APP, subFuture.resolver());            // 18  // 26
  ultra.subscribe("oneJob", process.env.GALAXY_JOB, subFutureJobs.resolver());        // 19  // 27
}                                                                                     // 20  // 28
                                                                                      // 21  // 29
var Apps;                                                                             // 22  // 30
var Jobs;                                                                             // 23  // 31
var Services;                                                                         // 24  // 32
var collectionFuture = new Future();                                                  // 25  // 33
                                                                                      // 26  // 34
Meteor.startup(function () {                                                          // 27  // 35
  var Mongo = Package.mongo.Mongo;                                                    // 28  // 36
  if (ultra) {                                                                        // 29  // 37
    Apps = new Mongo.Collection("apps", {                                             // 30  // 38
      connection: ultra                                                               // 31  // 39
    });                                                                               // 32  // 40
    Jobs = new Mongo.Collection("jobs", {                                             // 33  // 41
      connection: ultra                                                               // 34  // 42
    });                                                                               // 35  // 43
    Services = new Mongo.Collection('services', {                                     // 36  // 44
      connection: ultra                                                               // 37  // 45
    });                                                                               // 38  // 46
    // allow us to block on the collections being ready                               // 39  // 47
    collectionFuture.return();                                                        // 40  // 48
  }                                                                                   // 41  // 49
});                                                                                   // 42  // 50
                                                                                      // 43  // 51
// XXX: Remove this once we allow the same collection to be new'd from multiple       // 44  // 52
// places.                                                                            // 45  // 53
AppConfig._getAppCollection = function () {                                           // 46  // 54
  collectionFuture.wait();                                                            // 47  // 55
  return Apps;                                                                        // 48  // 56
};                                                                                    // 49  // 57
                                                                                      // 50  // 58
AppConfig._getJobsCollection = function () {                                          // 51  // 59
  collectionFuture.wait();                                                            // 52  // 60
  return Jobs;                                                                        // 53  // 61
};                                                                                    // 54  // 62
                                                                                      // 55  // 63
                                                                                      // 56  // 64
var staticAppConfig;                                                                  // 57  // 65
                                                                                      // 58  // 66
try {                                                                                 // 59  // 67
  if (process.env.APP_CONFIG) {                                                       // 60  // 68
    staticAppConfig = JSON.parse(process.env.APP_CONFIG);                             // 61  // 69
  } else {                                                                            // 62  // 70
    var settings;                                                                     // 63  // 71
    try {                                                                             // 64  // 72
      if (process.env.METEOR_SETTINGS) {                                              // 65  // 73
        settings = JSON.parse(process.env.METEOR_SETTINGS);                           // 66  // 74
      }                                                                               // 67  // 75
    } catch (e) {                                                                     // 68  // 76
      Log.warn("Could not parse METEOR_SETTINGS as JSON");                            // 69  // 77
    }                                                                                 // 70  // 78
    staticAppConfig = {                                                               // 71  // 79
      settings: settings,                                                             // 72  // 80
      packages: {                                                                     // 73  // 81
        'mongo-livedata': {                                                           // 74  // 82
          url: process.env.MONGO_URL,                                                 // 75  // 83
          oplog: process.env.MONGO_OPLOG_URL                                          // 76  // 84
        }                                                                             // 77  // 85
      }                                                                               // 78  // 86
    };                                                                                // 79  // 87
  }                                                                                   // 80  // 88
} catch (e) {                                                                         // 81  // 89
  Log.warn("Could not parse initial APP_CONFIG environment variable");                // 82  // 90
};                                                                                    // 83  // 91
                                                                                      // 84  // 92
AppConfig.getAppConfig = function () {                                                // 85  // 93
  if (!subFuture.isResolved() && staticAppConfig) {                                   // 86  // 94
    return staticAppConfig;                                                           // 87  // 95
  }                                                                                   // 88  // 96
  subFuture.wait();                                                                   // 89  // 97
  var myApp = Apps.findOne(process.env.GALAXY_APP);                                   // 90  // 98
  if (!myApp) {                                                                       // 91  // 99
    throw new Error("there is no app config for this app");                           // 92  // 100
  }                                                                                   // 93  // 101
  var config = myApp.config;                                                          // 94  // 102
  return config;                                                                      // 95  // 103
};                                                                                    // 96  // 104
                                                                                      // 97  // 105
AppConfig.getStarForThisJob = function () {                                           // 98  // 106
  if (ultra) {                                                                        // 99  // 107
    subFutureJobs.wait();                                                             // 100
    var job = Jobs.findOne(process.env.GALAXY_JOB);                                   // 101
    if (job) {                                                                        // 102
      return job.star;                                                                // 103
    }                                                                                 // 104
  }                                                                                   // 105
  return null;                                                                        // 106
};                                                                                    // 107
                                                                                      // 108
AppConfig.configurePackage = function (packageName, configure) {                      // 109
  var appConfig = AppConfig.getAppConfig(); // Will either be based in the env var,   // 110
                                         // or wait for galaxy to connect.            // 111
  var lastConfig =                                                                    // 112
        (appConfig && appConfig.packages &&                                           // 113
         appConfig.packages[packageName]) || {};                                      // 114
                                                                                      // 115
  // Always call the configure callback "soon" even if the initial configuration      // 116
  // is empty (synchronously, though deferred would be OK).                           // 117
  // XXX make sure that all callers of configurePackage deal well with multiple       // 118
  // callback invocations!  eg, email does not                                        // 119
  configure(lastConfig);                                                              // 120
  var configureIfDifferent = function (app) {                                         // 121
    if (!EJSON.equals(                                                                // 122
           app.config && app.config.packages && app.config.packages[packageName],     // 123
           lastConfig)) {                                                             // 124
      lastConfig = app.config.packages[packageName];                                  // 125
      configure(lastConfig);                                                          // 126
    }                                                                                 // 127
  };                                                                                  // 128
  var subHandle;                                                                      // 129
  var observed = new Future();                                                        // 130
                                                                                      // 131
  // This is not required to finish, so defer it so it doesn't block anything         // 132
  // else.                                                                            // 133
  Meteor.defer( function () {                                                         // 134
    // there's a Meteor.startup() that produces the various collections, make         // 135
    // sure it runs first before we continue.                                         // 136
    collectionFuture.wait();                                                          // 137
    subHandle = Apps.find(process.env.GALAXY_APP).observe({                           // 138
      added: configureIfDifferent,                                                    // 139
      changed: configureIfDifferent                                                   // 140
    });                                                                               // 141
    observed.return();                                                                // 142
  });                                                                                 // 143
                                                                                      // 144
  return {                                                                            // 145
    stop: function () {                                                               // 146
      observed.wait();                                                                // 147
      subHandle.stop();                                                               // 148
    }                                                                                 // 149
  };                                                                                  // 150
};                                                                                    // 151
                                                                                      // 152
AppConfig.configureService = function (serviceName, version, configure) {             // 153
                                                                                      // 154
  // Collect all the endpoints for this service, from both old- and new-format        // 155
  // documents, and call the `configure` callback with all the service endpoints      // 156
  // that we know about.                                                              // 157
  var callConfigure = function (doc) {                                                // 158
    var serviceDocs = Services.find({                                                 // 159
      name: serviceName,                                                              // 160
      version: version                                                                // 161
    });                                                                               // 162
    var endpoints = [];                                                               // 163
    serviceDocs.forEach(function (serviceDoc) {                                       // 164
      if (serviceDoc.providers) {                                                     // 165
        _.each(serviceDoc.providers, function (endpoint, app) {                       // 166
          endpoints.push(endpoint);                                                   // 167
        });                                                                           // 168
      } else {                                                                        // 169
        endpoints.push(serviceDoc.endpoint);                                          // 170
      }                                                                               // 171
    });                                                                               // 172
    configure(endpoints);                                                             // 173
  };                                                                                  // 174
                                                                                      // 175
  if (ultra) {                                                                        // 176
    // there's a Meteor.startup() that produces the various collections, make         // 177
    // sure it runs first before we continue.                                         // 178
    collectionFuture.wait();                                                          // 179
    // First try to subscribe to the new format service registrations; if that        // 180
    // sub doesn't exist, then ultraworld hasn't updated to the new format yet,       // 181
    // so try the old format `servicesByName` sub instead.                            // 182
    ultra.subscribe('services', serviceName, version, {                               // 183
      onError: function (err) {                                                       // 184
        if (err.error === 404) {                                                      // 185
          ultra.subscribe('servicesByName', serviceName);                             // 186
        }                                                                             // 187
      }                                                                               // 188
    });                                                                               // 189
    return Services.find({                                                            // 190
      name: serviceName,                                                              // 191
      version: version                                                                // 192
    }).observe({                                                                      // 193
      added: callConfigure,                                                           // 194
      changed: callConfigure,                                                         // 195
      removed: callConfigure                                                          // 196
    });                                                                               // 197
  }                                                                                   // 198
                                                                                      // 199
};                                                                                    // 200
                                                                                      // 201
////////////////////////////////////////////////////////////////////////////////////////     // 210
                                                                                             // 211
}).call(this);                                                                               // 212
                                                                                             // 213
///////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['application-configuration'] = {
  AppConfig: AppConfig
};

})();

//# sourceMappingURL=application-configuration.js.map
