(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var ECMAScript = Package.ecmascript.ECMAScript;
var Accounts = Package['accounts-base'].Accounts;
var AccountsServer = Package['accounts-base'].AccountsServer;
var SandstormDb = Package['sandstorm-db'].SandstormDb;
var SandstormPermissions = Package['sandstorm-permissions'].SandstormPermissions;
var babelHelpers = Package['babel-runtime'].babelHelpers;
var Symbol = Package['ecmascript-runtime'].Symbol;
var Map = Package['ecmascript-runtime'].Map;
var Set = Package['ecmascript-runtime'].Set;
var Promise = Package.promise.Promise;

/* Package-scope variables */
var inMeteor, promiseToFuture, waitPromise, SANDSTORM_ALTHOME, SandstormBackend;

(function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                     //
// packages/sandstorm-backend/sandstorm-backend.js                                                                     //
//                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                       //
// Sandstorm - Personal Cloud Sandbox                                                                                  //
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                                               //
// All rights reserved.                                                                                                //
//                                                                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");                                                     //
// you may not use this file except in compliance with the License.                                                    //
// You may obtain a copy of the License at                                                                             //
//                                                                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                                                                        //
//                                                                                                                     //
// Unless required by applicable law or agreed to in writing, software                                                 //
// distributed under the License is distributed on an "AS IS" BASIS,                                                   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                            //
// See the License for the specific language governing permissions and                                                 //
// limitations under the License.                                                                                      //
                                                                                                                       //
var Capnp = Npm.require("capnp");                                                                                      // 17
var Backend = Capnp.importSystem("sandstorm/backend.capnp").Backend;                                                   // 18
var Crypto = Npm.require("crypto");                                                                                    // 19
var Future = Npm.require("fibers/future");                                                                             // 20
var Promise = Npm.require("es6-promise").Promise;                                                                      // 21
                                                                                                                       //
var inMeteorInternal = Meteor.bindEnvironment(function (callback) {                                                    // 23
  callback();                                                                                                          // 24
});                                                                                                                    //
                                                                                                                       //
inMeteor = function (callback) {                                                                                       // 27
  // Calls the callback in a Meteor context.  Returns a Promise for its result.                                        //
  return new Promise(function (resolve, reject) {                                                                      // 29
    inMeteorInternal(function () {                                                                                     // 30
      try {                                                                                                            // 31
        resolve(callback());                                                                                           // 32
      } catch (err) {                                                                                                  //
        reject(err);                                                                                                   // 34
      }                                                                                                                //
    });                                                                                                                //
  });                                                                                                                  //
};                                                                                                                     //
                                                                                                                       //
promiseToFuture = function (promise) {                                                                                 // 40
  var result = new Future();                                                                                           // 41
  promise.then(result["return"].bind(result), result["throw"].bind(result));                                           // 42
  return result;                                                                                                       // 43
};                                                                                                                     //
                                                                                                                       //
waitPromise = function (promise) {                                                                                     // 46
  return promiseToFuture(promise).wait();                                                                              // 47
};                                                                                                                     //
                                                                                                                       //
SANDSTORM_ALTHOME = Meteor.settings && Meteor.settings.home;                                                           // 50
                                                                                                                       //
SandstormBackend = function (db, backendCap) {                                                                         // 52
  this._db = db;                                                                                                       // 53
  this._backendCap = backendCap;                                                                                       // 54
};                                                                                                                     //
                                                                                                                       //
SandstormBackend.prototype.cap = function () {                                                                         // 57
  return this._backendCap;                                                                                             // 58
};                                                                                                                     //
                                                                                                                       //
SandstormBackend.prototype.deleteUser = function (userId) {                                                            // 61
  return waitPromise(this._backendCap.deleteUser());                                                                   // 62
};                                                                                                                     //
                                                                                                                       //
SandstormBackend.prototype.shutdownGrain = function (grainId, ownerId, keepSessions) {                                 // 65
  if (!keepSessions) {                                                                                                 // 66
    Sessions.remove({ grainId: grainId });                                                                             // 67
  }                                                                                                                    //
                                                                                                                       //
  var grain = this._backendCap.getGrain(ownerId, grainId).supervisor;                                                  // 70
  return grain.shutdown().then(function () {                                                                           // 71
    grain.close();                                                                                                     // 72
    throw new Error("expected shutdown() to throw disconnected");                                                      // 73
  }, function (err) {                                                                                                  //
                                                                                                                       //
    grain.close();                                                                                                     // 76
    if (err.kjType !== "disconnected") {                                                                               // 77
      throw err;                                                                                                       // 78
    }                                                                                                                  //
  });                                                                                                                  //
};                                                                                                                     //
                                                                                                                       //
SandstormBackend.prototype.deleteGrain = function (grainId, ownerId) {                                                 // 83
  // We leave it up to the caller if they want to actually wait, but some don't so we report                           //
  // exceptions.                                                                                                       //
  return this._backendCap.deleteGrain(ownerId, grainId)["catch"](function (err) {                                      // 86
    console.error("problem deleting grain " + grainId + ":", err.message);                                             // 87
    throw err;                                                                                                         // 88
  });                                                                                                                  //
};                                                                                                                     //
                                                                                                                       //
SandstormBackend.shouldRestartGrain = function (error, retryCount) {                                                   // 92
  // Given an error thrown by an RPC call to a grain, return whether or not it makes sense to try                      //
  // to restart the grain and retry. `retryCount` is the number of times that the request has                          //
  // already gone through this cycle (should be zero for the first call).                                              //
                                                                                                                       //
  return error.kjType === "disconnected" && retryCount < 1;                                                            // 97
};                                                                                                                     //
                                                                                                                       //
SandstormBackend.prototype.maybeRetryUseGrain = function (grainId, cb, retryCount, err) {                              // 100
  var _this = this;                                                                                                    // 101
  if (SandstormBackend.shouldRestartGrain(err, retryCount)) {                                                          // 102
    return inMeteor(function () {                                                                                      // 103
      return cb(_this.continueGrain(grainId).supervisor)["catch"](_this.maybeRetryUseGrain.bind(_this, grainId, cb, retryCount + 1));
    });                                                                                                                //
  } else {                                                                                                             //
    throw err;                                                                                                         // 108
  }                                                                                                                    //
};                                                                                                                     //
                                                                                                                       //
SandstormBackend.prototype.useGrain = function (grainId, cb) {                                                         // 112
  var _this2 = this;                                                                                                   //
                                                                                                                       //
  // This will open a grain for you, handling restarts if needed, and call the passed function with                    //
  // the supervisor capability as the only parameter. The callback must return a promise that used                     //
  // the supervisor, so that we can check if a disconnect error occurred, and retry if possible.                       //
  // This function returns the same promise that your callback returns.                                                //
  //                                                                                                                   //
  // This function is NOT expected to be run in a meteor context.                                                      //
                                                                                                                       //
  return inMeteor(function () {                                                                                        // 120
    return cb(_this2.continueGrain(grainId).supervisor)["catch"](_this2.maybeRetryUseGrain.bind(_this2, grainId, cb, 0));
  });                                                                                                                  //
};                                                                                                                     //
                                                                                                                       //
SandstormBackend.prototype.continueGrain = function (grainId) {                                                        // 126
  var grain = Grains.findOne(grainId);                                                                                 // 127
  if (!grain) {                                                                                                        // 128
    throw new Meteor.Error(404, "Grain Not Found", "Grain ID: " + grainId);                                            // 129
  }                                                                                                                    //
                                                                                                                       //
  // If a DevPackage with the same app ID is currently active, we let it override the installed                        //
  // package, so that the grain runs using the dev app.                                                                //
  var devPackage = DevPackages.findOne({ appId: grain.appId });                                                        // 134
  var isDev = undefined;                                                                                               // 135
  var pkg = undefined;                                                                                                 // 136
  if (devPackage) {                                                                                                    // 137
    isDev = true;                                                                                                      // 138
    pkg = devPackage;                                                                                                  // 139
  } else {                                                                                                             //
    pkg = Packages.findOne(grain.packageId);                                                                           // 141
    if (!pkg) {                                                                                                        // 142
      throw new Meteor.Error(500, "Grain's package not installed", "Package ID: " + grain.packageId);                  // 143
    }                                                                                                                  //
  }                                                                                                                    //
                                                                                                                       //
  var manifest = pkg.manifest;                                                                                         // 148
  var packageId = pkg._id;                                                                                             // 149
                                                                                                                       //
  if (!("continueCommand" in manifest)) {                                                                              // 151
    throw new Meteor.Error(500, "Package manifest defines no continueCommand.", "Package ID: " + packageId);           // 152
  }                                                                                                                    //
                                                                                                                       //
  return this.startGrainInternal(packageId, grainId, grain.userId, manifest.continueCommand, false, isDev);            // 156
};                                                                                                                     //
                                                                                                                       //
SandstormBackend.prototype.startGrainInternal = function (packageId, grainId, ownerId, command, isNew, isDev) {        // 160
  // Starts the grain supervisor.  Must be executed in a Meteor context.  Blocks until grain is                        //
  // started. Returns a promise for an object containing two fields: `owner` (the ID of the owning                     //
  // user) and `supervisor` (the supervisor capability).                                                               //
                                                                                                                       //
  if (isUserExcessivelyOverQuota(Meteor.users.findOne(ownerId))) {                                                     // 165
    throw new Meteor.Error(402, "Cannot start grain because owner's storage is exhausted.\n" + "Please ask them to upgrade.");
  }                                                                                                                    //
                                                                                                                       //
  // Ugly: Stay backwards-compatible with old manifests that had "executablePath" and "args" rather                    //
  //   than just "argv".                                                                                               //
  if ("args" in command) {                                                                                             // 173
    if (!("argv" in command)) {                                                                                        // 174
      command.argv = command.args;                                                                                     // 175
    }                                                                                                                  //
                                                                                                                       //
    delete command.args;                                                                                               // 178
  }                                                                                                                    //
                                                                                                                       //
  if ("executablePath" in command) {                                                                                   // 181
    if (!("deprecatedExecutablePath" in command)) {                                                                    // 182
      command.deprecatedExecutablePath = command.executablePath;                                                       // 183
    }                                                                                                                  //
                                                                                                                       //
    delete command.executablePath;                                                                                     // 186
  }                                                                                                                    //
                                                                                                                       //
  return this._backendCap.startGrain(ownerId, grainId, packageId, command, isNew, isDev);                              // 189
};                                                                                                                     //
                                                                                                                       //
SandstormBackend.prototype.updateLastActive = function (grainId, userId, identityId) {                                 // 192
  // Update the lastActive date on the grain, any relevant API tokens, and the user,                                   //
  // and also update the user's storage usage.                                                                         //
                                                                                                                       //
  var storagePromise = undefined;                                                                                      // 196
  if (Meteor.settings["public"].quotaEnabled) {                                                                        // 197
    storagePromise = globalBackend._backendCap.getUserStorageUsage(userId);                                            // 198
  }                                                                                                                    //
                                                                                                                       //
  var now = new Date();                                                                                                // 201
  if (Grains.update(grainId, { $set: { lastUsed: now } }) === 0) {                                                     // 202
    // Grain must have been deleted. Ignore.                                                                           //
    return;                                                                                                            // 204
  }                                                                                                                    //
                                                                                                                       //
  if (userId) {                                                                                                        // 207
    Meteor.users.update(userId, { $set: { lastActive: now } });                                                        // 208
  }                                                                                                                    //
                                                                                                                       //
  if (identityId) {                                                                                                    // 211
    Meteor.users.update({ _id: identityId }, { $set: { lastActive: now } });                                           // 212
    // Update any API tokens that match this user/grain pairing as well                                                //
    ApiTokens.update({ grainId: grainId, "owner.user.identityId": identityId }, { $set: { "owner.user.lastUsed": now } });
  }                                                                                                                    //
                                                                                                                       //
  if (Meteor.settings["public"].quotaEnabled) {                                                                        // 218
    try {                                                                                                              // 219
      var ownerId = Grains.findOne(grainId).userId;                                                                    // 220
      var size = parseInt(waitPromise(storagePromise).size);                                                           // 221
      Meteor.users.update(ownerId, { $set: { storageUsage: size } });                                                  // 222
      // TODO(security): Consider actively killing grains if the user is excessively over quota?                       //
      //   Otherwise a constantly-active grain could consume arbitrary space without being stopped.                    //
    } catch (err) {                                                                                                    //
      if (err.kjType !== "unimplemented") {                                                                            // 226
        console.error("error getting user storage usage:", err.stack);                                                 // 227
      }                                                                                                                //
    }                                                                                                                  //
  }                                                                                                                    //
};                                                                                                                     //
                                                                                                                       //
function generateSessionId(grainId, userId, salt) {                                                                    // 233
  var sessionParts = [grainId, salt];                                                                                  // 234
  if (userId) {                                                                                                        // 235
    sessionParts.push(userId);                                                                                         // 236
  }                                                                                                                    //
                                                                                                                       //
  var sessionInput = sessionParts.join(":");                                                                           // 239
  return Crypto.createHash("sha256").update(sessionInput).digest("hex");                                               // 240
}                                                                                                                      //
                                                                                                                       //
SandstormBackend.prototype.openSessionInternal = function (grainId, userId, identityId, title, apiToken, cachedSalt) {
                                                                                                                       //
  // Start the grain if it is not running. This is an optimization: if we didn't start it here,                        //
  // it would start on the first request to the session host, but we'd like to get started before                      //
  // the round trip.                                                                                                   //
  var supervisor = this.continueGrain(grainId).supervisor;                                                             // 248
                                                                                                                       //
  this.updateLastActive(grainId, userId, identityId);                                                                  // 250
                                                                                                                       //
  cachedSalt = cachedSalt || Random.id(22);                                                                            // 252
  var sessionId = generateSessionId(grainId, userId, cachedSalt);                                                      // 253
  var session = Sessions.findOne({ _id: sessionId });                                                                  // 254
  if (session) {                                                                                                       // 255
    // TODO(someday): also do some more checks for anonymous sessions (sessions without a userId).                     //
    if (session.identityId && session.identityId !== identityId || session.grainId !== grainId) {                      // 257
      var e = new Meteor.Error(500, "Duplicate SessionId");                                                            // 259
      console.error(e);                                                                                                // 260
      throw e;                                                                                                         // 261
    } else {                                                                                                           //
      return { supervisor: supervisor,                                                                                 // 263
        methodResult: {                                                                                                // 264
          sessionId: session._id,                                                                                      // 265
          title: title,                                                                                                // 266
          grainId: grainId,                                                                                            // 267
          hostId: session.hostId,                                                                                      // 268
          tabId: session.tabId,                                                                                        // 269
          salt: cachedSalt                                                                                             // 270
        }                                                                                                              //
      };                                                                                                               //
    }                                                                                                                  //
  }                                                                                                                    //
                                                                                                                       //
  // TODO(someday): Allow caller to specify the parent session from which to inherit the tab ID, or                    //
  //   something.                                                                                                      //
                                                                                                                       //
  session = {                                                                                                          // 279
    _id: sessionId,                                                                                                    // 280
    grainId: grainId,                                                                                                  // 281
    hostId: Crypto.createHash("sha256").update(sessionId).digest("hex").slice(0, 32),                                  // 282
    tabId: Crypto.createHash("sha256").update("tab:").update(sessionId).digest("hex").slice(0, 32),                    // 283
    timestamp: new Date().getTime(),                                                                                   // 284
    hasLoaded: false                                                                                                   // 285
  };                                                                                                                   //
                                                                                                                       //
  if (userId) {                                                                                                        // 288
    session.identityId = identityId;                                                                                   // 289
    session.userId = userId;                                                                                           // 290
  } else if (apiToken) {                                                                                               //
    session.hashedToken = apiToken._id;                                                                                // 292
  } else {}                                                                                                            //
  // Must be old-style sharing, i.e. !grain.private.                                                                   //
  // jscs:ignore disallowEmptyBlocks                                                                                   //
                                                                                                                       //
  Sessions.insert(session);                                                                                            // 295
                                                                                                                       //
  return { supervisor: supervisor,                                                                                     // 299
    methodResult: {                                                                                                    // 300
      sessionId: session._id,                                                                                          // 301
      title: title,                                                                                                    // 302
      grainId: grainId,                                                                                                // 303
      hostId: session.hostId,                                                                                          // 304
      tabId: session.tabId,                                                                                            // 305
      salt: cachedSalt                                                                                                 // 306
    }                                                                                                                  //
  };                                                                                                                   //
};                                                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['sandstorm-backend'] = {
  SandstormBackend: SandstormBackend
};

})();

//# sourceMappingURL=sandstorm-backend.js.map
